@@ -16,21 +16,19 @@
 
 #include "php_filter.h"
 
-void php_filter_callback(PHP_INPUT_FILTER_PARAM_DECL)
+zend_result php_filter_callback(PHP_INPUT_FILTER_PARAM_DECL)
 {
 	zval retval;
-	zval args[1];
 	int status;
 
-	if (!option_array || !zend_is_callable(option_array, 0, NULL)) {
+	if (!option_array || !zend_is_callable(option_array, IS_CALLABLE_SUPPRESS_DEPRECATIONS, NULL)) {
 		zend_type_error("%s(): Option must be a valid callback", get_active_function_name());
 		zval_ptr_dtor(value);
 		ZVAL_NULL(value);
-		return;
+		return SUCCESS;
 	}
 
-	ZVAL_COPY(&args[0], value);
-	status = call_user_function(NULL, NULL, option_array, &retval, 1, args);
+	status = call_user_function(NULL, NULL, option_array, &retval, 1, value);
 
 	if (status == SUCCESS && !Z_ISUNDEF(retval)) {
 		zval_ptr_dtor(value);
@@ -39,6 +37,5 @@ void php_filter_callback(PHP_INPUT_FILTER_PARAM_DECL)
 		zval_ptr_dtor(value);
 		ZVAL_NULL(value);
 	}
-
-	zval_ptr_dtor(&args[0]);
+	return SUCCESS;
 }
