@@ -1,11 +1,9 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -28,36 +26,29 @@ extern "C" {
 #include "../calendar/calendar_class.h"
 }
 
-int datefmt_process_calendar_arg(zval* calendar_zv,
-								 Locale const& locale,
-								 const char *func_name,
-								 intl_error *err,
-								 Calendar*& cal,
-								 zend_long& cal_int_type,
-								 bool& calendar_owned)
-{
-	char *msg;
+using icu::GregorianCalendar;
+
+zend_result datefmt_process_calendar_arg(
+	zend_object *calendar_obj, zend_long calendar_long, bool calendar_is_null, Locale const& locale,
+	intl_error *err, Calendar*& cal, zend_long& cal_int_type, bool& calendar_owned
+) {
 	UErrorCode status = UErrorCode();
 
-	if (calendar_zv == NULL || Z_TYPE_P(calendar_zv) == IS_NULL) {
-
+	if (calendar_is_null) {
 		// default requested
 		cal = new GregorianCalendar(locale, status);
 		calendar_owned = true;
 
 		cal_int_type = UCAL_GREGORIAN;
 
-	} else if (Z_TYPE_P(calendar_zv) == IS_LONG) {
-
-		zend_long v = Z_LVAL_P(calendar_zv);
+	} else if (!calendar_obj) {
+		zend_long v = calendar_long;
 		if (v != (zend_long)UCAL_TRADITIONAL && v != (zend_long)UCAL_GREGORIAN) {
-			spprintf(&msg, 0, "%s: invalid value for calendar type; it must be "
-					"one of IntlDateFormatter::TRADITIONAL (locale's default "
-					"calendar) or IntlDateFormatter::GREGORIAN. "
-					"Alternatively, it can be an IntlCalendar object",
-					func_name);
-			intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, msg, 1);
-			efree(msg);
+			intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR,
+				"Invalid value for calendar type; it must be one of "
+				"IntlDateFormatter::TRADITIONAL (locale's default calendar) or"
+				" IntlDateFormatter::GREGORIAN. Alternatively, it can be an "
+				"IntlCalendar object");
 			return FAILURE;
 		} else if (v == (zend_long)UCAL_TRADITIONAL) {
 			cal = Calendar::createInstance(locale, status);
@@ -66,18 +57,12 @@ int datefmt_process_calendar_arg(zval* calendar_zv,
 		}
 		calendar_owned = true;
 
-		cal_int_type = Z_LVAL_P(calendar_zv);
+		cal_int_type = calendar_long;
 
-	} else if (Z_TYPE_P(calendar_zv) == IS_OBJECT &&
-			instanceof_function_ex(Z_OBJCE_P(calendar_zv),
-			Calendar_ce_ptr, 0)) {
-
-		cal = calendar_fetch_native_calendar(calendar_zv);
+	} else if (calendar_obj) {
+		cal = calendar_fetch_native_calendar(calendar_obj);
 		if (cal == NULL) {
-			spprintf(&msg, 0, "%s: Found unconstructed IntlCalendar object",
-					func_name);
-			intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, msg, 1);
-			efree(msg);
+			intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, "Found unconstructed IntlCalendar object");
 			return FAILURE;
 		}
 		calendar_owned = false;
@@ -85,10 +70,8 @@ int datefmt_process_calendar_arg(zval* calendar_zv,
 		cal_int_type = -1;
 
 	} else {
-		spprintf(&msg, 0, "%s: Invalid calendar argument; should be an integer "
-				"or an IntlCalendar instance", func_name);
-		intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, msg, 1);
-		efree(msg);
+		intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR,
+			"Invalid calendar argument; should be an integer or an IntlCalendar instance");
 		return FAILURE;
 	}
 
@@ -96,9 +79,7 @@ int datefmt_process_calendar_arg(zval* calendar_zv,
 		status = U_MEMORY_ALLOCATION_ERROR;
 	}
 	if (U_FAILURE(status)) {
-		spprintf(&msg, 0, "%s: Failure instantiating calendar", func_name);
-		intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, msg, 1);
-		efree(msg);
+		intl_errors_set(err, U_ILLEGAL_ARGUMENT_ERROR, "Failure instantiating calendar");
 		return FAILURE;
 	}
 
