@@ -1,5 +1,5 @@
 /* This is a generated file, edit the .stub.php file instead.
- * Stub hash: c416c231c5d1270b7e5961f84cc3ca3e29db4959 */
+ * Stub hash: a8de025fa96a78db3a26d53a18bb2b365d094eca */
 
 ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_opcache_reset, 0, 0, _IS_BOOL, 0)
 ZEND_END_ARG_INFO()
@@ -26,6 +26,8 @@ ZEND_END_ARG_INFO()
 
 #define arginfo_opcache_is_script_cached arginfo_opcache_compile_file
 
+#define arginfo_opcache_is_script_cached_in_file_cache arginfo_opcache_compile_file
+
 ZEND_FUNCTION(opcache_reset);
 ZEND_FUNCTION(opcache_get_status);
 ZEND_FUNCTION(opcache_compile_file);
@@ -33,6 +35,7 @@ ZEND_FUNCTION(opcache_invalidate);
 ZEND_FUNCTION(opcache_jit_blacklist);
 ZEND_FUNCTION(opcache_get_configuration);
 ZEND_FUNCTION(opcache_is_script_cached);
+ZEND_FUNCTION(opcache_is_script_cached_in_file_cache);
 
 static const zend_function_entry ext_functions[] = {
 	ZEND_FE(opcache_reset, arginfo_opcache_reset)
@@ -42,5 +45,6 @@ static const zend_function_entry ext_functions[] = {
 	ZEND_FE(opcache_jit_blacklist, arginfo_opcache_jit_blacklist)
 	ZEND_FE(opcache_get_configuration, arginfo_opcache_get_configuration)
 	ZEND_FE(opcache_is_script_cached, arginfo_opcache_is_script_cached)
+	ZEND_FE(opcache_is_script_cached_in_file_cache, arginfo_opcache_is_script_cached_in_file_cache)
 	ZEND_FE_END
 };
