@@ -13,9 +13,9 @@ function readline(?string $prompt = null): string|false {}
 /** @param int|string|bool|null $value */
 function readline_info(?string $var_name = null, $value = null): mixed {}
 
-function readline_add_history(string $prompt): bool {}
+function readline_add_history(string $prompt): true {}
 
-function readline_clear_history(): bool {}
+function readline_clear_history(): true {}
 
 #ifdef HAVE_HISTORY_LIST
 /**
@@ -33,7 +33,7 @@ function readline_completion_function(callable $callback): bool {}
 
 
 #ifdef HAVE_RL_CALLBACK_READ_CHAR
-function readline_callback_handler_install(string $prompt, callable $callback): bool {}
+function readline_callback_handler_install(string $prompt, callable $callback): true {}
 
 function readline_callback_read_char(): void {}
 
