@@ -64,6 +64,15 @@
 # else
 #  undef SO_BPF_EXTENSIONS
 # endif
+# if defined(HAVE_LINUX_IF_PACKET_H)
+#  include <linux/if_packet.h>
+# endif
+# if defined(HAVE_LINUX_IF_ETHER_H)
+#  include <linux/if_ether.h>
+# endif
+# if defined(HAVE_LINUX_UDP_H)
+#  include <linux/udp.h>
+# endif
 #endif
 
 #include <stddef.h>
@@ -91,6 +100,20 @@ ZEND_DECLARE_MODULE_GLOBALS(sockets)
 #define PF_INET AF_INET
 #endif
 
+#if defined(AF_PACKET)
+#define PHP_ETH_PROTO_CHECK(protocol, family)								\
+	do {												\
+		/* We ll let EINVAL errno warning about miusage, too many protocols conflicts */	\
+                if (protocol <= USHRT_MAX && family == AF_PACKET) {					\
+                    protocol = htons(protocol);								\
+                }											\
+        } while (0)
+#else
+#define PHP_ETH_PROTO_CHECK(protocol, family)								\
+	(void)protocol;											\
+	(void)family
+#endif
+
 static PHP_GINIT_FUNCTION(sockets);
 static PHP_GSHUTDOWN_FUNCTION(sockets);
 static PHP_MINIT_FUNCTION(sockets);
@@ -217,7 +240,7 @@ zend_module_entry sockets_module_entry = {
 ZEND_GET_MODULE(sockets)
 #endif
 
-static bool php_open_listen_sock(php_socket *sock, int port, int backlog) /* {{{ */
+static bool php_open_listen_sock(php_socket *sock, unsigned short port, int backlog) /* {{{ */
 {
 	struct sockaddr_in  la = {0};
 	struct hostent	    *hp;
@@ -232,7 +255,7 @@ static bool php_open_listen_sock(php_socket *sock, int port, int backlog) /* {{{
 
 	memcpy((char *) &la.sin_addr, hp->h_addr, hp->h_length);
 	la.sin_family = hp->h_addrtype;
-	la.sin_port = htons((unsigned short) port);
+	la.sin_port = htons(port);
 
 	sock->bsd_socket = socket(PF_INET, SOCK_STREAM, 0);
 	sock->blocking = 1;
@@ -262,6 +285,19 @@ static bool php_open_listen_sock(php_socket *sock, int port, int backlog) /* {{{
 
 static bool php_accept_connect(php_socket *in_sock, php_socket *out_sock, struct sockaddr *la, socklen_t *la_len) /* {{{ */
 {
+#if defined(HAVE_ACCEPT4)
+	int flags = SOCK_CLOEXEC;
+	if (!in_sock->blocking) {
+		flags |= SOCK_NONBLOCK;
+	}
+
+	out_sock->bsd_socket = accept4(in_sock->bsd_socket, la, la_len, flags);
+
+	if (IS_INVALID_SOCKET(out_sock)) {
+		PHP_SOCKET_ERROR(out_sock, "unable to accept incoming connection", errno);
+		return 0;
+	}
+#else
 	out_sock->bsd_socket = accept(in_sock->bsd_socket, la, la_len);
 
 	if (IS_INVALID_SOCKET(out_sock)) {
@@ -271,7 +307,7 @@ static bool php_accept_connect(php_socket *in_sock, php_socket *out_sock, struct
 
 #if !defined(PHP_WIN32)
 	/**
-	 * accept4 could had been used but not all platforms support it (e.g. Haiku, solaris < 11.4, ...)
+	 * for fewer and fewer platforms not supporting accept4 syscall we use fcntl instead,
 	 * win32, not having any concept of child process, has no need to address it.
 	 */
 	int mode;
@@ -289,6 +325,7 @@ static bool php_accept_connect(php_socket *in_sock, php_socket *out_sock, struct
 			return 0;
 		}
 	}
+#endif
 #endif
 
 	out_sock->error = 0;
@@ -680,10 +717,15 @@ PHP_FUNCTION(socket_create_listen)
 		Z_PARAM_LONG(backlog)
 	ZEND_PARSE_PARAMETERS_END();
 
+	if (port < 0 || port > USHRT_MAX) {
+		zend_argument_value_error(1, "must be between 0 and %u", USHRT_MAX);
+		RETURN_THROWS();
+	}
+
 	object_init_ex(return_value, socket_ce);
 	php_sock = Z_SOCKET_P(return_value);
 
-	if (!php_open_listen_sock(php_sock, port, backlog)) {
+	if (!php_open_listen_sock(php_sock, (unsigned short)port, backlog)) {
 		zval_ptr_dtor(return_value);
 		RETURN_FALSE;
 	}
@@ -955,13 +997,16 @@ PHP_FUNCTION(socket_read)
 /* {{{ Queries the remote side of the given socket which may either result in host/port or in a UNIX filesystem path, dependent on its type. */
 PHP_FUNCTION(socket_getsockname)
 {
-	zval					*arg1, *addr, *port = NULL;
+	zval					*arg1, *addr, *objint = NULL;
 	php_sockaddr_storage	sa_storage = {0};
 	php_socket				*php_sock;
 	struct sockaddr			*sa;
 	struct sockaddr_in		*sin;
 #ifdef HAVE_IPV6
 	struct sockaddr_in6		*sin6;
+#endif
+#ifdef AF_PACKET
+	struct sockaddr_ll              *sll;
 #endif
 	char					addrbuf[INET6_ADDRSTRLEN];
 	struct sockaddr_un		*s_un;
@@ -972,7 +1017,7 @@ PHP_FUNCTION(socket_getsockname)
 		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
 		Z_PARAM_ZVAL(addr)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_ZVAL(port)
+		Z_PARAM_ZVAL(objint)
 	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
@@ -992,8 +1037,8 @@ PHP_FUNCTION(socket_getsockname)
 			inet_ntop(AF_INET6, &sin6->sin6_addr,  addrbuf, sizeof(addrbuf));
 			ZEND_TRY_ASSIGN_REF_STRING(addr, addrbuf);
 
-			if (port != NULL) {
-				ZEND_TRY_ASSIGN_REF_LONG(port, htons(sin6->sin6_port));
+			if (objint != NULL) {
+				ZEND_TRY_ASSIGN_REF_LONG(objint, htons(sin6->sin6_port));
 			}
 			RETURN_TRUE;
 			break;
@@ -1003,8 +1048,8 @@ PHP_FUNCTION(socket_getsockname)
 			addr_string = inet_ntop(AF_INET, &sin->sin_addr, addrbuf, sizeof(addrbuf));
 			ZEND_TRY_ASSIGN_REF_STRING(addr, addr_string);
 
-			if (port != NULL) {
-				ZEND_TRY_ASSIGN_REF_LONG(port, htons(sin->sin_port));
+			if (objint != NULL) {
+				ZEND_TRY_ASSIGN_REF_LONG(objint, htons(sin->sin_port));
 			}
 			RETURN_TRUE;
 			break;
@@ -1015,9 +1060,26 @@ PHP_FUNCTION(socket_getsockname)
 			ZEND_TRY_ASSIGN_REF_STRING(addr, s_un->sun_path);
 			RETURN_TRUE;
 			break;
+#ifdef AF_PACKET
+		case AF_PACKET:
+			sll = (struct sockaddr_ll *) sa;
+			char ifrname[IFNAMSIZ];
+
+			if (UNEXPECTED(!if_indextoname(sll->sll_ifindex, ifrname))) {
+				zend_throw_error(NULL, "invalid interface index");
+				RETURN_THROWS();
+			}
+
+			ZEND_TRY_ASSIGN_REF_STRING(addr, ifrname);
+			if (objint != NULL) {
+				ZEND_TRY_ASSIGN_REF_LONG(objint, sll->sll_ifindex);
+			}
+			RETURN_TRUE;
+			break;
+#endif
 
 		default:
-			zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET, or AF_INET6");
+			zend_argument_value_error(1, "must be one of AF_UNIX, AF_PACKET, AF_INET, or AF_INET6");
 			RETURN_THROWS();
 	}
 }
@@ -1112,9 +1174,12 @@ PHP_FUNCTION(socket_create)
 	if (domain != AF_UNIX
 #ifdef HAVE_IPV6
 		&& domain != AF_INET6
+#endif
+#ifdef AF_PACKET
+		&& domain != AF_PACKET
 #endif
 		&& domain != AF_INET) {
-		zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET6, or AF_INET");
+		zend_argument_value_error(1, "must be one of AF_UNIX, AF_PACKET, AF_INET6, or AF_INET");
 		RETURN_THROWS();
 	}
 
@@ -1133,6 +1198,8 @@ PHP_FUNCTION(socket_create)
 		RETURN_THROWS();
 	}
 
+	PHP_ETH_PROTO_CHECK(protocol, domain);
+
 	object_init_ex(return_value, socket_ce);
 	php_sock = Z_SOCKET_P(return_value);
 
@@ -1156,15 +1223,14 @@ PHP_FUNCTION(socket_connect)
 {
 	zval				*resource_socket;
 	php_socket			*php_sock;
-	char				*addr;
+	zend_string                     *addr;
 	int					retval;
-	size_t              addr_len;
 	zend_long				port;
 	bool				port_is_null = 1;
 
 	ZEND_PARSE_PARAMETERS_START(2, 3)
 		Z_PARAM_OBJECT_OF_CLASS(resource_socket, socket_ce)
-		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_STR(addr)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_LONG_OR_NULL(port, port_is_null)
 	ZEND_PARSE_PARAMETERS_END();
@@ -1217,15 +1283,15 @@ PHP_FUNCTION(socket_connect)
 		case AF_UNIX: {
 			struct sockaddr_un s_un = {0};
 
-			if (addr_len >= sizeof(s_un.sun_path)) {
+			if (ZSTR_LEN(addr) >= sizeof(s_un.sun_path)) {
 				zend_argument_value_error(2, "must be less than %d", sizeof(s_un.sun_path));
 				RETURN_THROWS();
 			}
 
 			s_un.sun_family = AF_UNIX;
-			memcpy(&s_un.sun_path, addr, addr_len);
+			memcpy(&s_un.sun_path, ZSTR_VAL(addr), ZSTR_LEN(addr));
 			retval = connect(php_sock->bsd_socket, (struct sockaddr *) &s_un,
-				(socklen_t)(XtOffsetOf(struct sockaddr_un, sun_path) + addr_len));
+				(socklen_t)(XtOffsetOf(struct sockaddr_un, sun_path) + ZSTR_LEN(addr)));
 			break;
 		}
 
@@ -1268,21 +1334,25 @@ PHP_FUNCTION(socket_bind)
 	php_sockaddr_storage	sa_storage = {0};
 	struct sockaddr			*sock_type = (struct sockaddr*) &sa_storage;
 	php_socket				*php_sock;
-	char					*addr;
-	size_t						addr_len;
-	zend_long					port = 0;
+	zend_string                             *addr;
+	zend_long					objint = 0;
 	zend_long					retval = 0;
 
 	ZEND_PARSE_PARAMETERS_START(2, 3)
 		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
-		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_STR(addr)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_LONG(port)
+		Z_PARAM_LONG(objint)
 	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
 
+	if (objint < 0 || objint > USHRT_MAX) {
+		zend_argument_value_error(3, "must be between 0 and %u", USHRT_MAX);
+		RETURN_THROWS();
+	}
+
 	switch(php_sock->type) {
 		case AF_UNIX:
 			{
@@ -1290,14 +1360,14 @@ PHP_FUNCTION(socket_bind)
 
 				sa->sun_family = AF_UNIX;
 
-				if (addr_len >= sizeof(sa->sun_path)) {
+				if (ZSTR_LEN(addr) >= sizeof(sa->sun_path)) {
 					zend_argument_value_error(2, "must be less than %d", sizeof(sa->sun_path));
 					RETURN_THROWS();
 				}
-				memcpy(&sa->sun_path, addr, addr_len);
+				memcpy(&sa->sun_path, ZSTR_VAL(addr), ZSTR_LEN(addr));
 
 				retval = bind(php_sock->bsd_socket, (struct sockaddr *) sa,
-						offsetof(struct sockaddr_un, sun_path) + addr_len);
+						offsetof(struct sockaddr_un, sun_path) + ZSTR_LEN(addr));
 				break;
 			}
 
@@ -1306,7 +1376,7 @@ PHP_FUNCTION(socket_bind)
 				struct sockaddr_in *sa = (struct sockaddr_in *) sock_type;
 
 				sa->sin_family = AF_INET;
-				sa->sin_port = htons((unsigned short) port);
+				sa->sin_port = htons((unsigned short) objint);
 
 				if (! php_set_inet_addr(sa, addr, php_sock)) {
 					RETURN_FALSE;
@@ -1321,7 +1391,7 @@ PHP_FUNCTION(socket_bind)
 				struct sockaddr_in6 *sa = (struct sockaddr_in6 *) sock_type;
 
 				sa->sin6_family = AF_INET6;
-				sa->sin6_port = htons((unsigned short) port);
+				sa->sin6_port = htons((unsigned short) objint);
 
 				if (! php_set_inet6_addr(sa, addr, php_sock)) {
 					RETURN_FALSE;
@@ -1330,9 +1400,26 @@ PHP_FUNCTION(socket_bind)
 				retval = bind(php_sock->bsd_socket, (struct sockaddr *)sa, sizeof(struct sockaddr_in6));
 				break;
 			}
+#endif
+#ifdef AF_PACKET
+		case AF_PACKET:
+			{
+				struct sockaddr_ll *sa = (struct sockaddr_ll *) sock_type;
+				socklen_t sa_len = sizeof(sa);
+
+				if (getsockname(php_sock->bsd_socket, sock_type, &sa_len) < 0) {
+					zend_value_error("invalid AF_PACKET socket");
+					RETURN_THROWS();
+				}
+
+				sa->sll_ifindex = if_nametoindex(ZSTR_VAL(addr));
+
+				retval = bind(php_sock->bsd_socket, sock_type, sizeof(struct sockaddr_ll));
+				break;
+			}
 #endif
 		default:
-			zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET, or AF_INET6");
+			zend_argument_value_error(1, "must be one of AF_UNIX, AF_PACKET, AF_INET, or AF_INET6");
 			RETURN_THROWS();
 	}
 
@@ -1433,6 +1520,9 @@ PHP_FUNCTION(socket_recvfrom)
 	struct sockaddr_in	sin;
 #ifdef HAVE_IPV6
 	struct sockaddr_in6	sin6;
+#endif
+#ifdef AF_PACKET
+	//struct sockaddr_ll     sll;
 #endif
 	char				addrbuf[INET6_ADDRSTRLEN];
 	socklen_t			slen;
@@ -1537,6 +1627,38 @@ PHP_FUNCTION(socket_recvfrom)
 			ZEND_TRY_ASSIGN_REF_STRING(arg5, addrbuf[0] ? addrbuf : "::");
 			ZEND_TRY_ASSIGN_REF_LONG(arg6, ntohs(sin6.sin6_port));
 			break;
+#endif
+#ifdef AF_PACKET
+			/*
+		case AF_PACKET:
+			// TODO expose and use proper ethernet frame type instead i.e. src mac, dst mac and payload to userland
+			// ditto for socket_sendto
+			slen = sizeof(sll);
+			memset(&sll, 0, sizeof(sll));
+			sll.sll_family = AF_PACKET;
+			char ifrname[IFNAMSIZ];
+
+			retval = recvfrom(php_sock->bsd_socket, ZSTR_VAL(recv_buf), arg3, arg4, (struct sockaddr *)&sll, (socklen_t *)&slen);
+
+			if (retval < 0) {
+				PHP_SOCKET_ERROR(php_sock, "unable to recvfrom", errno);
+				zend_string_efree(recv_buf);
+				RETURN_FALSE;
+			}
+			ZSTR_LEN(recv_buf) = retval;
+			ZSTR_VAL(recv_buf)[ZSTR_LEN(recv_buf)] = '\0';
+
+			if (UNEXPECTED(!if_indextoname(sll.sll_ifindex, ifrname))) {
+				PHP_SOCKET_ERROR(php_sock, "unable to get the interface name", errno);
+				zend_string_efree(recv_buf);
+				RETURN_FALSE;
+			}
+
+			ZEND_TRY_ASSIGN_REF_NEW_STR(arg2, recv_buf);
+			ZEND_TRY_ASSIGN_REF_STRING(arg5, ifrname);
+			ZEND_TRY_ASSIGN_REF_LONG(arg6, sll.sll_ifindex);
+			break;
+			*/
 #endif
 		default:
 			zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET, or AF_INET6");
@@ -1556,19 +1678,23 @@ PHP_FUNCTION(socket_sendto)
 	struct sockaddr_in	sin;
 #ifdef HAVE_IPV6
 	struct sockaddr_in6	sin6;
+#endif
+#ifdef AF_PACKET
+	//struct sockaddr_ll      sll;
 #endif
 	int					retval;
-	size_t              buf_len, addr_len;
-	zend_long			len, flags, port;
+	size_t              buf_len;
+	zend_long			len, flags, port = 0;
 	bool           port_is_null = 1;
-	char				*buf, *addr;
+	char				*buf;
+	zend_string *addr;
 
 	ZEND_PARSE_PARAMETERS_START(5, 6)
 		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
 		Z_PARAM_STRING(buf, buf_len)
 		Z_PARAM_LONG(len)
 		Z_PARAM_LONG(flags)
-		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_STR(addr)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_LONG_OR_NULL(port, port_is_null)
 	ZEND_PARSE_PARAMETERS_END();
@@ -1576,6 +1702,12 @@ PHP_FUNCTION(socket_sendto)
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
 
+	if (port < 0 || port > USHRT_MAX) {
+		zend_argument_value_error(6, "must be between 0 and %u", USHRT_MAX);
+		RETURN_THROWS();
+	}
+
+
 	if (len < 0) {
 		zend_argument_value_error(3, "must be greater than or equal to 0");
 		RETURN_THROWS();
@@ -1585,7 +1717,7 @@ PHP_FUNCTION(socket_sendto)
 		case AF_UNIX:
 			memset(&s_un, 0, sizeof(s_un));
 			s_un.sun_family = AF_UNIX;
-			snprintf(s_un.sun_path, sizeof(s_un.sun_path), "%s", addr);
+			snprintf(s_un.sun_path, sizeof(s_un.sun_path), "%s", ZSTR_VAL(addr));
 
 			retval = sendto(php_sock->bsd_socket, buf, ((size_t)len > buf_len) ? buf_len : (size_t)len,	flags, (struct sockaddr *) &s_un, SUN_LEN(&s_un));
 			break;
@@ -1623,6 +1755,22 @@ PHP_FUNCTION(socket_sendto)
 
 			retval = sendto(php_sock->bsd_socket, buf, ((size_t)len > buf_len) ? buf_len : (size_t)len, flags, (struct sockaddr *) &sin6, sizeof(sin6));
 			break;
+#endif
+#ifdef AF_PACKET
+			/*
+		case AF_PACKET:
+			if (port_is_null) {
+				zend_argument_value_error(6, "cannot be null when the socket type is AF_PACKET");
+				RETURN_THROWS();
+			}
+
+			memset(&sll, 0, sizeof(sll));			
+			sll.sll_family = AF_PACKET;
+			sll.sll_ifindex = port;
+
+			retval = sendto(php_sock->bsd_socket, buf, ((size_t)len > buf_len) ? buf_len : (size_t)len, flags, (struct sockaddr *) &sin, sizeof(sin));
+			break;
+		*/
 #endif
 		default:
 			zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET, or AF_INET6");
@@ -1645,7 +1793,9 @@ PHP_FUNCTION(socket_get_option)
 	struct linger	linger_val;
 	struct timeval	tv;
 #ifdef PHP_WIN32
-	int				timeout = 0;
+	DWORD				timeout = 0;
+#else
+	struct timeval timeout;
 #endif
 	socklen_t		optlen;
 	php_socket		*php_sock;
@@ -1707,6 +1857,29 @@ PHP_FUNCTION(socket_get_option)
 			}
 		}
 #endif
+
+#ifdef TCP_FUNCTION_BLK
+		case TCP_FUNCTION_BLK:
+#ifdef TCP_FUNCTION_ALIAS
+		case TCP_FUNCTION_ALIAS:
+#endif
+		{
+
+			struct tcp_function_set tsf = {0};
+			optlen = sizeof(tsf);
+
+			if (getsockopt(php_sock->bsd_socket, level, optname, &tsf, &optlen) != 0) {
+				PHP_SOCKET_ERROR(php_sock, "Unable to retrieve socket option", errno);
+				RETURN_FALSE;
+			}
+
+			array_init_size(return_value, 2);
+
+			add_assoc_string(return_value, "function_set_name", tsf.function_set_name);
+			add_assoc_long(return_value, "pcbcnt", tsf.pcbcnt);
+			return;
+		}
+#endif
 		}
 	}
 
@@ -1723,33 +1896,29 @@ PHP_FUNCTION(socket_get_option)
 					RETURN_FALSE;
 				}
 
-				array_init(return_value);
+				array_init_size(return_value, 2);
 				add_assoc_long(return_value, "l_onoff", linger_val.l_onoff);
 				add_assoc_long(return_value, "l_linger", linger_val.l_linger);
 				return;
 
 			case SO_RCVTIMEO:
 			case SO_SNDTIMEO:
-#ifndef PHP_WIN32
-				optlen = sizeof(tv);
-
-				if (getsockopt(php_sock->bsd_socket, level, optname, (char*)&tv, &optlen) != 0) {
-					PHP_SOCKET_ERROR(php_sock, "Unable to retrieve socket option", errno);
-					RETURN_FALSE;
-				}
-#else
-				optlen = sizeof(int);
+				optlen = sizeof(timeout);
 
 				if (getsockopt(php_sock->bsd_socket, level, optname, (char*)&timeout, &optlen) != 0) {
 					PHP_SOCKET_ERROR(php_sock, "Unable to retrieve socket option", errno);
 					RETURN_FALSE;
 				}
 
-				tv.tv_sec = timeout ? timeout / 1000 : 0;
-				tv.tv_usec = timeout ? (timeout * 1000) % 1000000 : 0;
+#ifndef PHP_WIN32
+				tv.tv_sec = timeout.tv_sec;
+				tv.tv_usec = timeout.tv_usec;
+#else
+				tv.tv_sec = timeout ? (long)(timeout / 1000) : 0;
+				tv.tv_usec = timeout ? (long)((timeout % 1000) * 1000) : 0;
 #endif
 
-				array_init(return_value);
+				array_init_size(return_value, 2);
 
 				add_assoc_long(return_value, "sec", tv.tv_sec);
 				add_assoc_long(return_value, "usec", tv.tv_usec);
@@ -1771,7 +1940,7 @@ PHP_FUNCTION(socket_get_option)
 					RETURN_FALSE;
 				}
 
-				array_init(return_value);
+				array_init_size(return_value, 9);
 
 				add_assoc_long(return_value, "rmem_alloc", minfo[SK_MEMINFO_RMEM_ALLOC]);
 				add_assoc_long(return_value, "rcvbuf", minfo[SK_MEMINFO_RCVBUF]);
@@ -1796,7 +1965,7 @@ PHP_FUNCTION(socket_get_option)
 					RETURN_FALSE;
 				}
 
-				array_init(return_value);
+				array_init_size(return_value, 1);
 
 				add_assoc_string(return_value, "af_name", af.af_name);
 				return;
@@ -1820,9 +1989,12 @@ PHP_FUNCTION(socket_get_option)
 					RETURN_FALSE;
 				}
 
-				array_init(return_value);
+				size_t arrlen = optlen / sizeof(struct fil_info);
 
-				for (i = 0; i < optlen / sizeof(struct fil_info); i++) {
+				array_init_size(return_value, arrlen);
+				zend_hash_real_init_packed(Z_ARRVAL_P(return_value));
+
+				for (i = 0; i < arrlen; i++) {
 					add_index_string(return_value, i, fi[i].fi_name);
 				}
 
@@ -1855,15 +2027,13 @@ PHP_FUNCTION(socket_set_option)
 	php_socket				*php_sock;
 	int						ov, optlen, retval;
 #ifdef PHP_WIN32
-	int						timeout;
+	DWORD						timeout;
 #else
 	struct					timeval tv;
 #endif
 	zend_long					level, optname;
 	void 					*opt_ptr;
 	HashTable		 		*opt_ht;
-	zval 					*l_onoff, *l_linger;
-	zval		 			*sec, *usec;
 
 	ZEND_PARSE_PARAMETERS_START(4, 4)
 		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
@@ -1904,13 +2074,12 @@ PHP_FUNCTION(socket_set_option)
 		switch (optname) {
 #ifdef TCP_CONGESTION
 		case TCP_CONGESTION: {
-			if (Z_TYPE_P(arg4) == IS_STRING) {
-				opt_ptr = Z_STRVAL_P(arg4);
-				optlen = Z_STRLEN_P(arg4);
-			} else {
-				opt_ptr = "";
-				optlen = 0;
+			if (Z_TYPE_P(arg4) != IS_STRING) {
+				zend_argument_type_error(4, "must be of type string when argument #3 ($option) is TCP_CONGESTION, %s given", zend_zval_value_name(arg4));
+				RETURN_THROWS();
 			}
+			opt_ptr = Z_STRVAL_P(arg4);
+			optlen = Z_STRLEN_P(arg4);
 			if (setsockopt(php_sock->bsd_socket, level, optname, opt_ptr, optlen) != 0) {
 				PHP_SOCKET_ERROR(php_sock, "Unable to set socket option", errno);
 				RETURN_FALSE;
@@ -1919,6 +2088,39 @@ PHP_FUNCTION(socket_set_option)
 			RETURN_TRUE;
 		}
 #endif
+
+#ifdef TCP_FUNCTION_BLK
+		case TCP_FUNCTION_BLK: {
+			if (Z_TYPE_P(arg4) != IS_STRING) {
+				zend_argument_type_error(4, "must be of type string when argument #3 ($option) is TCP_FUNCTION_BLK, %s given", zend_zval_value_name(arg4));
+				RETURN_THROWS();
+			}
+			if (zend_str_has_nul_byte(Z_STR_P(arg4))) {
+				zend_argument_value_error(4, "must not contain null bytes when argument #3 ($option) is TCP_FUNCTION_BLK");
+				RETURN_THROWS();
+			}
+			/* [...] the unique name of the TCP stack, and should be no longer than
+			 * TCP_FUNCTION_NAME_LEN_MAX-1 characters in length.
+			 * https://github.com/freebsd/freebsd-src/blob/da2c88dfcf4f425e6e0a58d6df3a7c8e88d8df92/share/man/man9/tcp_functions.9#L193C1-L194C50 */
+			if (Z_STRLEN_P(arg4) >= TCP_FUNCTION_NAME_LEN_MAX) {
+				zend_argument_value_error(4, "must be less than %zu bytes when argument #3 ($option) is TCP_FUNCTION_BLK", TCP_FUNCTION_NAME_LEN_MAX);
+				RETURN_THROWS();
+			}
+			struct tcp_function_set tfs = {0};
+			/* Copy the trailing nul byte */
+			memcpy(tfs.function_set_name, Z_STRVAL_P(arg4), Z_STRLEN_P(arg4)+1);
+
+			optlen = sizeof(tfs);
+			opt_ptr = &tfs;
+			if (setsockopt(php_sock->bsd_socket, level, optname, opt_ptr, optlen) != 0) {
+				PHP_SOCKET_ERROR(php_sock, "Unable to set socket option", errno);
+				RETURN_FALSE;
+			}
+
+			RETURN_TRUE;
+		}
+#endif
+
 		}
 	}
 
@@ -1929,6 +2131,7 @@ PHP_FUNCTION(socket_set_option)
 		case SO_LINGER: {
 			const char l_onoff_key[] = "l_onoff";
 			const char l_linger_key[] = "l_linger";
+			zval *l_onoff, *l_linger;
 
 			if (Z_TYPE_P(arg4) != IS_ARRAY) {
 				if (UNEXPECTED(Z_TYPE_P(arg4) != IS_OBJECT)) {
@@ -1975,6 +2178,7 @@ PHP_FUNCTION(socket_set_option)
 		case SO_SNDTIMEO: {
 			const char sec_key[] = "sec";
 			const char usec_key[] = "usec";
+			zval *sec, *usec;
 
 			if (Z_TYPE_P(arg4) != IS_ARRAY) {
 				if (UNEXPECTED(Z_TYPE_P(arg4) != IS_OBJECT)) {
@@ -2006,22 +2210,37 @@ PHP_FUNCTION(socket_set_option)
 			optlen = sizeof(tv);
 			opt_ptr = &tv;
 #else
-			timeout = valsec * 1000 + valusec / 1000;
-	
-			optlen = sizeof(int);
+			if (valsec < 0 || valsec > ULONG_MAX / 1000) {
+				zend_argument_value_error(4, "\"%s\" must be between 0 and %u", sec_key, (ULONG_MAX / 1000));
+				RETURN_THROWS();
+			}
+
+			timeout = valsec * 1000;
+
+
+			/*
+			 * We deliberately throw if (valusec / 1000) > ULONG_MAX, treating it as a programmer error.
+			 * On Windows, ULONG_MAX = 2^32, unlike ZEND_LONG_MAX = 2^63.
+			 */
+			if (valusec < 0 || timeout > ULONG_MAX - (valusec / 1000)) {
+				zend_argument_value_error(4, "\"%s\" must be between 0 and %u", usec_key, (DWORD)(ULONG_MAX - (valusec / 1000)));
+				RETURN_THROWS();
+			}
+
+			timeout += valusec / 1000;
+			optlen = sizeof(timeout);
 			opt_ptr = &timeout;
 #endif
 			break;
 		}
 #ifdef SO_BINDTODEVICE
 		case SO_BINDTODEVICE: {
-			if (Z_TYPE_P(arg4) == IS_STRING) {
-				opt_ptr = Z_STRVAL_P(arg4);
-				optlen = Z_STRLEN_P(arg4);
-			} else {
-				opt_ptr = "";
-				optlen = 0;
+			if (Z_TYPE_P(arg4) != IS_STRING) {
+				zend_argument_type_error(4, "must be of type string when argument #3 ($option) is SO_BINDTODEVICE, %s given", zend_zval_value_name(arg4));
+				RETURN_THROWS();
 			}
+			opt_ptr = Z_STRVAL_P(arg4);
+			optlen = Z_STRLEN_P(arg4);
 			break;
 		}
 #endif
@@ -2029,11 +2248,20 @@ PHP_FUNCTION(socket_set_option)
 #ifdef SO_ACCEPTFILTER
 		case SO_ACCEPTFILTER: {
 			if (Z_TYPE_P(arg4) != IS_STRING) {
-				php_error_docref(NULL, E_WARNING, "Invalid filter argument type");
-				RETURN_FALSE;
+				zend_argument_type_error(4, "must be of type string when argument #3 ($option) is SO_ACCEPTFILTER, %s given", zend_zval_value_name(arg4));
+				RETURN_THROWS();
+			}
+			if (zend_str_has_nul_byte(Z_STR_P(arg4))) {
+				zend_argument_value_error(4, "must not contain null bytes when argument #3 ($option) is SO_ACCEPTFILTER");
+				RETURN_THROWS();
 			}
 			struct accept_filter_arg af = {0};
-			strlcpy(af.af_name, Z_STRVAL_P(arg4), sizeof(af.af_name));
+			if (Z_STRLEN_P(arg4) >= sizeof(af.af_name)) {
+				zend_argument_value_error(4, "must be less than %zu bytes when argument #3 ($option) is SO_ACCEPTFILTER", sizeof(af.af_name));
+				RETURN_THROWS();
+			}
+			/* Copy the trailing nul byte */
+			memcpy(af.af_name, Z_STRVAL_P(arg4), Z_STRLEN_P(arg4)+1);
 			opt_ptr = &af;
 			optlen = sizeof(af);
 			break;
@@ -2048,8 +2276,8 @@ PHP_FUNCTION(socket_set_option)
 				RETURN_FALSE;
 			}
 			if (Z_TYPE_P(arg4) != IS_STRING) {
-				php_error_docref(NULL, E_WARNING, "Invalid filter argument type");
-				RETURN_FALSE;
+				zend_argument_type_error(4, "must be of type string when argument #3 ($option) is FIL_ATTACH or FIL_DETACH, %s given", zend_zval_value_name(arg4));
+				RETURN_THROWS();
 			}
 			opt_ptr = Z_STRVAL_P(arg4);
 			optlen = Z_STRLEN_P(arg4);
@@ -2092,6 +2320,22 @@ PHP_FUNCTION(socket_set_option)
 		}
 #endif
 
+#if defined(UDP_SEGMENT)
+		case UDP_SEGMENT: {
+			ov = zval_get_long(arg4);
+
+			// UDP segmentation offload maximum size or 0 to disable it
+			if (ov < 0 || ov > USHRT_MAX) {
+				zend_argument_value_error(4, "must be of between 0 and %u", USHRT_MAX);
+				RETURN_FALSE;
+			}
+
+			optlen = sizeof(ov);
+			opt_ptr = &ov;
+			break;
+		}
+#endif
+
 		default:
 default_case:
 			ov = zval_get_long(arg4);
@@ -2151,27 +2395,23 @@ PHP_FUNCTION(socket_create_pair)
 		RETURN_THROWS();
 	}
 
+	if (socketpair(domain, type, protocol, fds_array) != 0) {
+		SOCKETS_G(last_error) = errno;
+		php_error_docref(NULL, E_WARNING, "Unable to create socket pair [%d]: %s", errno, sockets_strerror(errno));
+		RETURN_FALSE;
+	}
+
+	fds_array_zval = zend_try_array_init_size(fds_array_zval, 2);
+	if (!fds_array_zval) {
+		RETURN_THROWS();
+	}
+
 	object_init_ex(&retval[0], socket_ce);
 	php_sock[0] = Z_SOCKET_P(&retval[0]);
 
 	object_init_ex(&retval[1], socket_ce);
 	php_sock[1] = Z_SOCKET_P(&retval[1]);
 
-	if (socketpair(domain, type, protocol, fds_array) != 0) {
-		SOCKETS_G(last_error) = errno;
-		php_error_docref(NULL, E_WARNING, "Unable to create socket pair [%d]: %s", errno, sockets_strerror(errno));
-		zval_ptr_dtor(&retval[0]);
-		zval_ptr_dtor(&retval[1]);
-		RETURN_FALSE;
-	}
-
-	fds_array_zval = zend_try_array_init(fds_array_zval);
-	if (!fds_array_zval) {
-		zval_ptr_dtor(&retval[0]);
-		zval_ptr_dtor(&retval[1]);
-		RETURN_THROWS();
-	}
-
 	php_sock[0]->bsd_socket = fds_array[0];
 	php_sock[1]->bsd_socket = fds_array[1];
 	php_sock[0]->type		= domain;
@@ -2206,6 +2446,11 @@ PHP_FUNCTION(socket_shutdown)
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
 
+	if (how_shutdown < SHUT_RD || how_shutdown > SHUT_RDWR) {
+		zend_argument_value_error(2, "must be one of SHUT_RD, SHUT_WR or SHUT_RDWR");
+		RETURN_THROWS();
+	}
+
 	if (shutdown(php_sock->bsd_socket, how_shutdown) != 0) {
 		PHP_SOCKET_ERROR(php_sock, "Unable to shutdown socket", errno);
 		RETURN_FALSE;
@@ -2342,7 +2587,7 @@ PHP_FUNCTION(socket_import_stream)
 	ZEND_PARSE_PARAMETERS_END();
 	php_stream_from_zval(stream, zstream);
 
-	if (php_stream_cast(stream, PHP_STREAM_AS_SOCKETD, (void**)&socket, 1)) {
+	if (php_stream_cast(stream, PHP_STREAM_AS_SOCKETD, (void**)&socket, 1) == FAILURE) {
 		/* error supposedly already shown */
 		RETURN_FALSE;
 	}
@@ -2490,22 +2735,83 @@ PHP_FUNCTION(socket_addrinfo_lookup)
 
 	memset(&hints, 0, sizeof(hints));
 
-	if (zhints && !HT_IS_PACKED(Z_ARRVAL_P(zhints))) {
+#if defined(PHP_WIN32)
+#  if !defined(AF_MAX)
+#    define AF_MAX (AF_BTH + 1)
+#  endif
+#endif
+
+	if (zhints) {
+		if (UNEXPECTED(HT_IS_PACKED(Z_ARRVAL_P(zhints)))) {
+			zend_argument_value_error(3, "must only contain array keys \"ai_flags\", \"ai_socktype\", "
+					"\"ai_protocol\", or \"ai_family\"");
+			RETURN_THROWS();
+		}
+
 		ZEND_HASH_MAP_FOREACH_STR_KEY_VAL(Z_ARRVAL_P(zhints), key, hint) {
 			if (key) {
+				bool failed = false;
 				if (zend_string_equals_literal(key, "ai_flags")) {
-					hints.ai_flags = zval_get_long(hint);
+					zend_long val = zval_try_get_long(hint, &failed);
+					if (failed) {
+						zend_argument_type_error(3, "\"ai_flags\" key must be of type int, %s given", zend_zval_type_name(hint));
+						RETURN_THROWS();
+					}
+					if (val < 0 || val > INT_MAX) {
+						zend_argument_value_error(3, "\"ai_flags\" key must be between 0 and %d", INT_MAX);
+						RETURN_THROWS();
+					}
+					hints.ai_flags = (int)val;
 				} else if (zend_string_equals_literal(key, "ai_socktype")) {
-					hints.ai_socktype = zval_get_long(hint);
+					zend_long val = zval_try_get_long(hint, &failed);
+					if (failed) {
+						zend_argument_type_error(3, "\"ai_socktype\" key must be of type int, %s given", zend_zval_type_name(hint));
+						RETURN_THROWS();
+					}
+					if (val < 0 || val > INT_MAX) {
+						zend_argument_value_error(3, "\"ai_socktype\" key must be between 0 and %d", INT_MAX);
+						RETURN_THROWS();
+					}
+					hints.ai_socktype = (int)val;
 				} else if (zend_string_equals_literal(key, "ai_protocol")) {
-					hints.ai_protocol = zval_get_long(hint);
+					zend_long val = zval_try_get_long(hint, &failed);
+					if (failed) {
+						zend_argument_type_error(3, "\"ai_protocol\" key must be of type int, %s given", zend_zval_type_name(hint));
+						RETURN_THROWS();
+					}
+					if (val < 0 || val > INT_MAX) {
+						zend_argument_value_error(3, "\"ai_protocol\" key must be between 0 and %d", INT_MAX);
+						RETURN_THROWS();
+					}
+					hints.ai_protocol = (int)val;
 				} else if (zend_string_equals_literal(key, "ai_family")) {
-					hints.ai_family = zval_get_long(hint);
+					zend_long val = zval_try_get_long(hint, &failed);
+					if (failed) {
+						zend_argument_type_error(3, "\"ai_family\" key must be of type int, %s given", zend_zval_type_name(hint));
+						RETURN_THROWS();
+					}
+					// Some platforms support also PF_LOCAL/AF_UNIX (e.g. FreeBSD) but the security concerns implied
+					// make it not worth handling it (e.g. unwarranted write permissions on the socket).
+					// Note existing socket_addrinfo* api already forbid such case.
+#ifdef HAVE_IPV6
+					if (val != AF_INET && val != AF_INET6) {
+						zend_argument_value_error(3, "\"ai_family\" key must be AF_INET or AF_INET6");
+#else
+					if (val != AF_INET) {
+						zend_argument_value_error(3, "\"ai_family\" key must be AF_INET");
+#endif
+						RETURN_THROWS();
+					}
+					hints.ai_family = (int)val;
 				} else {
 					zend_argument_value_error(3, "must only contain array keys \"ai_flags\", \"ai_socktype\", "
-						"\"ai_protocol\", or \"ai_family\"");
+							"\"ai_protocol\", or \"ai_family\"");
 					RETURN_THROWS();
-				}
+					}
+			} else {
+				zend_argument_value_error(3, "must only contain array keys \"ai_flags\", \"ai_socktype\", "
+						"\"ai_protocol\", or \"ai_family\"");
+				RETURN_THROWS();
 			}
 		} ZEND_HASH_FOREACH_END();
 	}
@@ -2515,6 +2821,7 @@ PHP_FUNCTION(socket_addrinfo_lookup)
 	}
 
 	array_init(return_value);
+	zend_hash_real_init_packed(Z_ARRVAL_P(return_value));
 
 	for (rp = result; rp != NULL; rp = rp->ai_next) {
 		if (rp->ai_family != AF_UNSPEC) {
@@ -2544,7 +2851,6 @@ PHP_FUNCTION(socket_addrinfo_lookup)
 PHP_FUNCTION(socket_addrinfo_bind)
 {
 	zval			*arg1;
-	int				retval;
 	php_addrinfo	*ai;
 	php_socket		*php_sock;
 
@@ -2554,6 +2860,10 @@ PHP_FUNCTION(socket_addrinfo_bind)
 
 	ai = Z_ADDRESS_INFO_P(arg1);
 
+	ZEND_ASSERT(ai->addrinfo.ai_family == AF_INET || ai->addrinfo.ai_family == AF_INET6);
+
+	PHP_ETH_PROTO_CHECK(ai->addrinfo.ai_protocol, ai->addrinfo.ai_family);
+
 	object_init_ex(return_value, socket_ce);
 	php_sock = Z_SOCKET_P(return_value);
 
@@ -2570,30 +2880,7 @@ PHP_FUNCTION(socket_addrinfo_bind)
 	php_sock->error = 0;
 	php_sock->blocking = 1;
 
-	switch(php_sock->type) {
-		case AF_UNIX:
-			{
-				// AF_UNIX sockets via getaddrino are not implemented due to security problems
-				close(php_sock->bsd_socket);
-				zval_ptr_dtor(return_value);
-				RETURN_FALSE;
-			}
-
-		case AF_INET:
-#ifdef HAVE_IPV6
-		case AF_INET6:
-#endif
-			{
-				retval = bind(php_sock->bsd_socket, ai->addrinfo.ai_addr, ai->addrinfo.ai_addrlen);
-				break;
-			}
-		default:
-			close(php_sock->bsd_socket);
-			zend_argument_value_error(1, "must be one of AF_UNIX, AF_INET, or AF_INET6");
-			RETURN_THROWS();
-	}
-
-	if (retval != 0) {
+	if (bind(php_sock->bsd_socket, ai->addrinfo.ai_addr, ai->addrinfo.ai_addrlen) != 0) {
 		PHP_SOCKET_ERROR(php_sock, "Unable to bind address", errno);
 		close(php_sock->bsd_socket);
 		zval_ptr_dtor(return_value);
@@ -2606,7 +2893,6 @@ PHP_FUNCTION(socket_addrinfo_bind)
 PHP_FUNCTION(socket_addrinfo_connect)
 {
 	zval			*arg1;
-	int				retval;
 	php_addrinfo	*ai;
 	php_socket		*php_sock;
 
@@ -2616,6 +2902,10 @@ PHP_FUNCTION(socket_addrinfo_connect)
 
 	ai = Z_ADDRESS_INFO_P(arg1);
 
+	ZEND_ASSERT(ai->addrinfo.ai_family == AF_INET || ai->addrinfo.ai_family == AF_INET6);
+
+	PHP_ETH_PROTO_CHECK(ai->addrinfo.ai_protocol, ai->addrinfo.ai_family);
+
 	object_init_ex(return_value, socket_ce);
 	php_sock = Z_SOCKET_P(return_value);
 
@@ -2632,30 +2922,7 @@ PHP_FUNCTION(socket_addrinfo_connect)
 	php_sock->error = 0;
 	php_sock->blocking = 1;
 
-	switch(php_sock->type) {
-		case AF_UNIX:
-			{
-				// AF_UNIX sockets via getaddrino are not implemented due to security problems
-				close(php_sock->bsd_socket);
-				zval_ptr_dtor(return_value);
-				RETURN_FALSE;
-			}
-
-		case AF_INET:
-#ifdef HAVE_IPV6
-		case AF_INET6:
-#endif
-			{
-				retval = connect(php_sock->bsd_socket, ai->addrinfo.ai_addr, ai->addrinfo.ai_addrlen);
-				break;
-			}
-		default:
-			zend_argument_value_error(1, "socket type must be one of AF_UNIX, AF_INET, or AF_INET6");
-			close(php_sock->bsd_socket);
-			RETURN_THROWS();
-	}
-
-	if (retval != 0) {
+	if (connect(php_sock->bsd_socket, ai->addrinfo.ai_addr, ai->addrinfo.ai_addrlen) != 0) {
 		PHP_SOCKET_ERROR(php_sock, "Unable to connect address", errno);
 		close(php_sock->bsd_socket);
 		zval_ptr_dtor(return_value);
